#!/usr/bin/env python

import pygame
import math
import random

use_sound = True
pygame.init()
#pygame.font.init()

try:
    pygame.mixer.init()
except:
    use_sound = False

pygame.display.set_caption("Rolling Hills")
screen = pygame.display.set_mode((640, 480))

font = pygame.font.Font("PhaistosDisk_-_Layne_Lighthouse.ttf", 24)

class MySpr(pygame.sprite.Sprite):
    def __init__(self, image=None, x=0, y=0):
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.x = x
        self.y = y
        self.dx = 0.0
        self.dy = 0.0
        if img is not None:
            self.ox = -self.image.get_width() / 2
            self.oy = -self.image.get_height()
            self.rect = self.image.get_rect()
            self.collision = pygame.sprite.Sprite()
            self.collision.rect = pygame.Rect(0,0,self.rect[2],self.rect[3])
            self.collision.ox = self.ox
            self.collision.oy = self.oy
        else:
            self.ox = 0
            self.oy = 0
            self.rect = pygame.Rect(0,0,0,0)
            self.collision = pygame.sprite.Sprite()
            self.collision.rect = pygame.Rect(0,0,0,0)
            self.collision.ox = 0
            self.collision.oy = 0
    
    def update_rects(self):
        self.rect[0] = int(self.x) + self.ox
        self.rect[1] = int(self.y) + self.oy
        
        self.collision.rect[0] = int(self.x) + self.collision.ox
        self.collision.rect[1] = int(self.y) + self.collision.oy
        
    def update(self, *args):
        self.y = land.get_at(self.x)
        self.update_rects()
        
    def draw(self, surf):
        surf.blit(self.image, self.rect)

class RollSpr(MySpr):
    def __init__(self, image=None, x=0):
        MySpr.__init__(self, image, x)
    
    def update(self, *args):
        ox = self.x
        self.x += self.dx
        if not (0 < self.x < 640):
            self.dx = -self.dx
            self.x = ox
        
        self.y = land.get_at(self.x)
        a = land.get_at(self.x - 1)
        b = land.get_at(self.x + 1)
        
        self.dx += -self.dx * 0.1
        if a < self.y:
            self.dx += (self.y - a)
        
        if b < self.y:
            self.dx -= (self.y - b)
        
        self.update_rects()
        
class Player(MySpr):
    def __init__(self, image=None, x=0):
        MySpr.__init__(self, img['ball'], 320)
        self.state = 0
        self.delay = 0
    
    def update(self, *args):
        global score, score_img, lives, last_oneup, state
        keys = args[0]
        
        if self.state == 0:
            if keys[pygame.K_LEFT]:
                self.dx -= 0.4
            
            if keys[pygame.K_RIGHT]:
                self.dx += 0.4
            
            ox = self.x
            self.x += self.dx
            if not (0 < self.x < 640):
                self.dx = -self.dx
                self.x = ox
            
            self.y = land.get_at(self.x)
            a = land.get_at(self.x - 1) * 0.2
            b = land.get_at(self.x + 1) * 0.2
            
            self.dx += -self.dx * 0.05
            if a < self.y:
                self.dx += (self.y - a)
            
            if b < self.y:
                self.dx -= (self.y - b)
                
            tmp = pygame.sprite.spritecollide(self, collectables, True)
            if len(tmp) > 0:
                if use_sound:
                    snd['ding'].play()
                score += len(tmp)
                score_img = font.render("Score: %i" % score, True, (0,0,0))
                if score > last_oneup + 50:
                    lives += 1
                    last_oneup += 50
                    if use_sound:
                        snd['oneup'].play()
                    
            if pygame.sprite.spritecollideany(self, hittables):
                self.image = img['pop']
                if use_sound:
                    snd['pop'].play()
                lives -= 1
                if lives > 0:
                    self.state = 1
                    self.delay = 60 * 3
                else:
                    self.state = 2
                    state = 3
                    score_img = font.render("Score: %i" % score, True, (255,255,255))
                    
        elif self.state == 1:
            self.delay -= 1
            if self.delay == 0:
                reset_level()
        
        self.update_rects()
        
class Diamond(MySpr):
    def __init__(self):
        MySpr.__init__(self, img['diamond'], random.randint(0, 640), 250)
    
    def update(self, *args):
        self.update_rects()
        
class Arrow(MySpr):
    def __init__(self):
        MySpr.__init__(self, img['arrow'], random.randint(0, 640), -10)
        hittables.add(self)
        
    def update(self, *args):
        self.y += 5.0
        if self.y > 500:
            hittables.remove(self)
        self.update_rects()

class Wave:
    def __init__(self):
        self.amp = (random.random() - 0.5) * 10.0 + 20.0
        self.freq = random.random() * 0.05
        self.dx = (random.random() - 0.5) * 2.0
        self.x = random.random() * math.pi * 2.0
    
    def update(self):
        self.x += self.dx
        
    def get_at(self, x):
        return self.amp * math.sin((self.x + x) * self.freq)

class Land:
    def __init__(self):
        self.waves = [Wave() for i in xrange(5)]
        self.offset = 320
    
    def update(self):
        for w in self.waves:
            w.update()
            
    def get_at(self, x):
        y = 0.0
        for w in self.waves:
            y += w.get_at(x)
            
        return y + self.offset
        
    def draw(self, surf):
        oy = self.offset
        for x in xrange(640):
            y = self.get_at(x)
            pygame.draw.line(surf, (0, 128, 255), (x, y), (x, 480), 1)
            pygame.draw.line(surf, (0, 0, 0), (x-1, oy), (x, y), 1)
            oy = y

def split_image(img, x, y):
	tmp = []
	w = img.get_width() / x
	h = img.get_height() / y
	
	for j in xrange(y):
		for i in xrange(x):
			tmp.append(img.subsurface(pygame.Rect(i*w, j*h, w, h)))
	return tmp

img = {}
#img['guy'] = pygame.image.load('guy.png').convert_alpha()
img['title'] = pygame.image.load('title.png')
img['end'] = pygame.image.load('end.png')
img['tree'] = pygame.image.load('tree.png').convert_alpha()
img['ball'] = pygame.image.load('ball.png').convert_alpha()
img['pop'] = pygame.image.load('pop.png').convert_alpha()
img['diamond'] = pygame.image.load('diamond.png').convert_alpha()
img['arrow'] = pygame.image.load('arrow.png').convert_alpha()
img['life'] = pygame.image.load('life.png').convert_alpha()

#img_list = {}
#img_list['guy'] = split_image(img['guy'], 4, 1)

snd = {}
if use_sound:
    snd['ding'] = pygame.mixer.Sound('ding.wav')
    snd['pop'] = pygame.mixer.Sound('pop.wav')
    snd['start'] = pygame.mixer.Sound('start.wav')
    snd['oneup'] = pygame.mixer.Sound('oneup.wav')
    pygame.mixer.music.load('music.ogg')
    pygame.mixer.music.play(-1)

def reset_level(sound = True):
    global land, sprites, player, collectables, hittables
    land = Land()
    collectables = pygame.sprite.Group()
    hittables = pygame.sprite.Group()
    sprites = pygame.sprite.Group()
    for t in xrange(5):
        sprites.add(MySpr(img['tree'], random.randint(0, 640)))
    #    sprites.add(RollSpr(img['ball'], random.randint(0, 640)))
    
    for d in xrange(10):
        collectables.add(Diamond())
        
    player = Player()
    
    if use_sound and sound:
        snd['start'].play()

def reset(sound = False):
    global score, score_img, lives, last_oneup
    reset_level(sound)
    score = 0
    score_img = font.render("Score: %i" % score, True, (0,0,0))
    lives = 3
    last_oneup = 0

reset()

clock = pygame.time.Clock()
state = 1
while state != 0:
    key = pygame.key.get_pressed()
    if state == 1:
        # TITLE
        screen.blit(img['title'], (0,0))
        
    elif state == 2:
        # GAME
        if len(collectables) == 0:
            reset_level()
            
        screen.fill((200,255,128))
        land.update()
        
        if random.random() < 0.01:
            Arrow()
        
        for s in sprites:
            s.update()
            s.draw(screen)
            
        for s in collectables:
            s.update()
            s.draw(screen)
            
        for s in hittables:
            s.update()
            s.draw(screen)
            
        player.update(key)
        player.draw(screen)
        
        land.draw(screen)
        
        screen.blit(score_img, (5,5))
        
        for i in xrange(lives):
            screen.blit(img['life'], (640 - img['life'].get_width() * (i + 1), 5))
        
    elif state == 3:
        # WIN
        screen.blit(img['end'], (0,0))
        screen.blit(score_img, (250,280))
    
    elif state == 4:
        #LOSE
        pass
    
    clock.tick(60)
    pygame.display.flip()
    
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            state = 0
            
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_ESCAPE:
                state = 0
                
            if event.key == pygame.K_SPACE:
                if state == 1:
                    state = 2
                    reset(True)
                
                elif state == 3:
                    state = 1

pygame.quit()
